#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
 Create instance
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

datastore = {
    'type': 'DDS-Community',
    'version': '3.4',
    'storage_engine': 'wiredTiger'
}
region = 'eu-de'
availability_zone = 'eu-de-01'
vpc_id = 'vpc-1'
subnet_id = 'subnet-1'
security_group_id = 'sg-1'
password = 'Test@123!'
mode = 'Sharding'
flavor = [
    {
        "type": "mongos",
        "num": 2,
        "spec_code": "dds.mongodb.s2.medium.4.mongos"
    },
    {
        "type": "shard",
        "num": 2,
        "storage": "ULTRAHIGH",
        "size": 20,
        "spec_code": "dds.mongodb.s2.medium.4.shard"
    },
    {
        "type": "config",
        "num": 1,
        "storage": "ULTRAHIGH",
        "size": 20,
        "spec_code": "dds.mongodb.s2.large.2.config"
    }
]
backup_strategy = {
    'start_time': '23:00-00:00',
    'keep_days': '8'
}
instance = conn.dds.create_instance(name='test-dds',
                                    datastore=datastore,
                                    region=region,
                                    availability_zone=availability_zone,
                                    vpc_id=vpc_id,
                                    subnet_id=subnet_id,
                                    security_group_id=security_group_id,
                                    password=password,
                                    mode=mode,
                                    flavor=flavor,
                                    backup_strategy=backup_strategy)
print(instance)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update custom authorizer
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
attrs = {
    'name': 'custom_auth_test',
    'type': 'BACKEND',
    'authorizer_type': 'FUNC',
    'authorizer_uri': 'urn:fss:eu-de:7ed5f793b8354ea9b27a849f17af4733'
                      ':function:default:test_apig_authorizer:latest',
    'authorizer_version': '1',
    'ttl': 5,
    "identities": [{
        "name": "header",
        "location": "HEADER"
    }]
}
updated = conn.apig.update_custom_authorizer(gateway='gateway_id',
                                             custom_authorizer='authorizer_id',
                                             **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
'''Instance v3 action implementations'''
import argparse

from osc_lib.cli import format_columns
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


_formatters = {
    'backup_strategy': format_columns.DictColumn,
    'charge_info': format_columns.DictColumn,
    'datastore': format_columns.DictColumn,
    'ha': format_columns.DictColumn,
    'nodes': format_columns.ListDictColumn,
    'volume': format_columns.DictColumn,
}


def _get_columns(item):
    column_map = {}
    hidden = ['job_id', 'location']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


def set_attributes_for_print(instances):
    for instance in instances:
        if getattr(instance, 'volume', None):
            setattr(instance, 'size', instance.volume['size'])
        else:
            setattr(instance, 'size', '-')
        datastore = instance.datastore
        if datastore.get('version'):
            setattr(instance, 'datastore_version', datastore['version'])
        if datastore.get('type'):
            setattr(instance, 'datastore_type', datastore['type'])
        yield instance


HA_MODE_CHOICES = ['sync', 'semisync', 'async']
DISK_TYPE_CHOICES = ['common', 'ultrahigh']
HA_TYPE_CHOICES = ['ha', 'replica', 'single']
DATASTORE_TYPE_CHOICES = ['mysql', 'postgresql', 'sqlserver']


class ListDatabaseInstances(command.Lister):
    _description = _('List database instances')
    columns = (
        'ID', 'Name', 'Datastore Type', 'Datastore Version', 'Status',
        'Flavor_ref', 'Type', 'Size', 'Region'
    )

    def get_parser(self, prog_name):
        parser = super(ListDatabaseInstances, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            help=_('Limit the number of results displayed'))
        parser.add_argument(
            '--id',
            dest='id',
            metavar='<id>',
            type=str,
            help=_('Specifies the DB instance ID.'))
        parser.add_argument(
            '--name',
            dest='name',
            metavar='<name>',
            type=str,
            help=_('Specifies the DB instance Name.'))
        parser.add_argument(
            '--type',
            dest='type',
            metavar='{' + ','.join(HA_TYPE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=HA_TYPE_CHOICES,
            help=_(
                'Specifies the instance type. '
                'Values cane be single/ha/replica.'))
        parser.add_argument(
            '--datastore-type',
            metavar='{' + ','.join(DATASTORE_TYPE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=DATASTORE_TYPE_CHOICES,
            help=_(
                'Specifies the database type. '
                'value is MySQL, PostgreSQL, or SQLServer.'))
        parser.add_argument(
            '--router-id',
            dest='router_id',
            metavar='<router_id>',
            type=str,
            help=_('Specifies the ID of Router to which '
                   'instance should be connected to.'))
        parser.add_argument(
            '--network-id',
            dest='network_id',
            metavar='<net_id>',
            type=str,
            help=_('Indicates the Neutron network ID of DB Instance.'))
        parser.add_argument(
            '--offset',
            dest='offset',
            metavar='<offset>',
            type=int,
            default=None,
            help=_('Specifies the index position.'))

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        args_list = [
            'name', 'id', 'router_id', 'network_id', 'offset', 'limit'
        ]
        attrs = {}
        for arg in args_list:
            if getattr(parsed_args, arg):
                attrs[arg] = getattr(parsed_args, arg)
        if parsed_args.type:
            attrs['type'] = parsed_args.type.title()
        if parsed_args.datastore_type == 'mysql':
            attrs['datastore_type'] = 'MySQL'
        elif parsed_args.datastore_type == 'postgresql':
            attrs['datastore_type'] = 'PostgreSQL'
        elif parsed_args.datastore_type == 'sqlserver':
            attrs['datastore_type'] = 'SQLServer'
        if parsed_args.limit:
            attrs['paginated'] = False

        data = client.instances(**attrs)
        if data:
            data = set_attributes_for_print(data)

        return (self.columns, (utils.get_item_properties(
            s,
            self.columns,
        ) for s in data))


class ShowDatabaseInstance(command.ShowOne):
    _description = _("Show instance details")

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Instance (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        obj = client.find_instance(parsed_args.instance, ignore_missing=False)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateDatabaseInstance(command.ShowOne):
    _description = _("Create a new database instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the instance.")
        )
        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            help=_("Flavor spec_code")
        )
        disk_group = parser.add_argument_group('Disk data')
        disk_group.add_argument(
            '--size',
            metavar='<size>',
            dest='volume_size',
            type=int,
            required=True,
            help=_("Size of the instance disk volume in GB. ")
        )
        disk_group.add_argument(
            '--volume-type',
            metavar='{' + ','.join(DISK_TYPE_CHOICES) + '}',
            type=lambda s: s.upper(),
            required=True,
            dest='volume_type',
            choices=[s.upper() for s in DISK_TYPE_CHOICES],
            help=_("Volume type. (COMMON, ULTRAHIGH).")
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            required=True,
            help=_("The Zone hint to give to Nova.")
        )
        parser.add_argument(
            '--region',
            metavar='<region>',
            required=True,
            help=argparse.SUPPRESS,
        )
        ds_group = parser.add_argument_group('Datasoure parameters')
        ds_group.add_argument(
            '--datastore-type',
            metavar='<datastore>',
            help=_("Name of the datastore (type).")
        )
        ds_group.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            help=_("Datastore version.")
        )
        parser.add_argument(
            '--configuration',
            dest='configuration',
            metavar='<configuration_id>',
            default=None,
            help=_("ID of the configuration group to attach to the instance.")
        )
        parser.add_argument(
            '--disk-encryption-id',
            metavar='<disk_encryption_id>',
            default=None,
            help=_("key ID for disk encryption.")
        )
        new_instance_group = parser.add_argument_group(
            'New instance parameters',
            'Parameters to be used for the new instance creation (not when '
            'created as replica or from backup')
        new_instance_group.add_argument(
            '--port',
            metavar='<port>',
            type=int,
            help=_("Database Port")
        )
        new_instance_group.add_argument(
            '--password',
            metavar='<password>',
            help=_("ID of the configuration group to attach to the instance.")
        )
        new_instance_group.add_argument(
            '--router-id',
            metavar='<router_id>',
            dest='router',
            help=_('ID of a Router the DB should be connected to')
        )
        new_instance_group.add_argument(
            '--network-id',
            metavar='<net_id>',
            dest='network',
            help=_('ID of a Neutron network the DB should be connected to.')
        )
        new_instance_group.add_argument(
            '--security-group-id',
            dest='security_group',
            metavar='<security_group_id>',
            help=_('Security group ID.')
        )
        parser.add_argument(
            '--ha-mode',
            metavar='{' + ','.join(HA_MODE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=HA_MODE_CHOICES,
            help=_('replication mode for the standby DB instance. '
                   'This parameter is required when using `ha` flavors.')
        )
        parser.add_argument(
            '--charge-mode',
            metavar='<charge_mode>',
            default='postPaid',
            help=_('Specifies the billing mode.')
        )
        create_from_group = parser.add_argument_group(
            'Create FROM group',
            'Parameters to be used when creating new instance as a '
            'replica or from backup')
        create_from_group.add_argument(
            '--replica-of',
            metavar='<source_instance>',
            default=None,
            help=_("ID or name of an existing instance to replicate from.")
        )
        create_from_group.add_argument(
            '--from-instance',
            metavar='<source_instance>',
            help=_('Source instance ID or Name to create from. '
                   'It is required when recovering from backup or PITR.')
        )
        create_from_group.add_argument(
            '--backup',
            metavar='<backup>',
            help=_('Backup ID or Name to create new instance from.')
        )
        create_from_group.add_argument(
            '--backup-keepdays',
            metavar='<backup_keepday>',
            help=_('Specifies the retention days for specific backup files.')
        )
        create_from_group.add_argument(
            '--backup-timeframe',
            metavar='<HH:MM-HH:MM>',
            help=_('Specifies the backup time window. Automated backups will '
                   'be triggered during the backup time window.')
        )
        create_from_group.add_argument(
            '--restore-time',
            metavar='<time>',
            help=_('Time (UNIX timestamp in ms) to create new instance '
                   'as a PointInTime recovery.')
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )
        parser.add_argument(
            '--wait-timeout',
            type=int,
            help=_('Timeout')
        )

        return parser

    def take_action(self, parsed_args):
        # Attention: not conform password result in BadRequest with no info

        # initialize sdk_connection with rds methods
        _ = self.app.client_manager.rds
        client = self.app.client_manager.sdk_connection
        attrs = {}
        for attr in [
            'availability_zone', 'backup', 'backup_keepdays',
            'backup_timeframe', 'charge_mode', 'configuration',
            'datastore_type', 'datastore_version', 'disk_encryption_id',
            'flavor', 'from_instance', 'ha_mode', 'name', 'network',
            'password', 'port', 'region', 'replica_of', 'restore_time',
            'router', 'security_group', 'volume_size', 'volume_type', 'wait',
            'wait_timeout', 'wait_interval'
        ]:
            if getattr(parsed_args, attr):
                attrs[attr] = getattr(parsed_args, attr)

        if not parsed_args.wait:
            attrs['wait'] = False

        obj = client.create_rds_instance(
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)


class DeleteDatabaseInstance(command.Command):

    _description = _("Deletes an instance.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the Instance'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=_('Wait for the instance to be deleted')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )
        parser.add_argument(
            '--wait-timeout',
            type=int,
            help=_('Timeout')
        )
        return parser

    def take_action(self, parsed_args):
        # initialize sdk_connection with rds methods
        _ = self.app.client_manager.rds
        client = self.app.client_manager.sdk_connection
        attrs = {}
        for attr in [
            'instance', 'wait', 'wait_timeout', 'wait_interval'
        ]:
            if getattr(parsed_args, attr):
                attrs[attr] = getattr(parsed_args, attr)

        if not parsed_args.wait:
            attrs['wait'] = False

        try:
            client.delete_rds_instance(**attrs)
        except Exception as e:
            msg = (_("Failed to delete instance %(instance)s: %(e)s") % {
                'instance': parsed_args.name,
                'e': e
            })
            raise exceptions.CommandError(msg)


class RestoreDatabaseInstance(command.Command):

    _description = _("Restores an instance from backup.")

    def get_parser(self, prog_name):
        parser = super(RestoreDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the target instance.')
        )
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            '--backup',
            metavar='<backup>',
            default=None,
            type=str,
            help=_('ID or name of the backup.')
        )
        group.add_argument(
            '--restore-time',
            metavar='<restore_time>',
            default=None,
            type=str,
            help=_('Specifies the time point of data '
                   'restoration in the UNIX timestamp')
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)
        backup = None
        if parsed_args.backup:
            backup = client.find_backup(name_or_id=parsed_args.backup,
                                        instance=instance,
                                        ignore_missing=False)

        obj = client.restore_instance(instance=instance,
                                      backup=backup,
                                      restore_time=parsed_args.restore_time)

        if obj.job_id and parsed_args.wait:
            wait_args = {}
            if parsed_args.wait_interval:
                wait_args['interval'] = parsed_args.wait_interval

            client.wait_for_job(obj.job_id, **wait_args)


class ShowBackupPolicy(command.ShowOne):
    _description = _('Show Database instance Backup Policy')

    def get_parser(self, prog_name):
        parser = super(ShowBackupPolicy, self).get_parser(prog_name)
        parser.add_argument('instance',
                            metavar='<instance>',
                            help=_('Instance ID or Name'))
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.rds

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)
        obj = client.get_instance_backup_policy(instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(obj, columns)

        return (display_columns, data)


class SetBackupPolicy(command.Command):
    _description = _('Set Database Backup Policy')

    def get_parser(self, prog_name):
        parser = super(SetBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Instance ID or Name'))
        parser.add_argument(
            '--keep-days',
            metavar='<keep_days>',
            type=int,
            required=True,
            help=_('Specifies the number of days to'
                   'retain the generated backup files.'))
        parser.add_argument(
            '--start-time',
            metavar='<start_time>',
            help=_('Specifies the backup time window.'
                   'The value must be a valid value in the'
                   '"hh:mm-HH:MM" format.'))
        parser.add_argument(
            '--period',
            metavar='<period>',
            help=_('Specifies the backup cycle'
                   'configuration. Data will be'
                   'automatically backed up on the'
                   'selected days every week.'))
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.rds
        args_list = ['keep_days', 'start_time', 'period']
        attrs = {}
        for arg in args_list:
            if getattr(parsed_args, arg) is not None:
                attrs[arg] = getattr(parsed_args, arg)

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)

        client.set_instance_backup_policy(instance, **attrs)


class ShowAvailableRestoreTime(command.Lister):
    _description = _('Show Database instance recovery timeframe')

    columns = ('start_time', 'end_time')
    column_headers = ('Start time', 'End time')

    def get_parser(self, prog_name):
        parser = super(ShowAvailableRestoreTime, self).get_parser(prog_name)
        parser.add_argument('instance',
                            metavar='<instance>',
                            help=_('Instance ID or Name'))
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.rds

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)
        data = client.get_instance_restore_time(instance)

        return (self.column_headers, (utils.get_dict_properties(
            s,
            self.columns,
        ) for s in data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestCluster(base.BaseFunctionalTest):
    TEST_CLUSTER = '5a66a449-668c-492f-8c33-5cdbdeaadd2e'

    def setUp(self):
        super(TestCluster, self).setUp()
        self.cce = self.conn.cce

    def test_list(self):

        objects = list(self.cce.clusters())

        for obj in objects:
            self.assertIsNotNone(obj.id)

        self.assertGreaterEqual(len(objects), 0)

    def test_get(self):
        obj = self.cce.get_cluster(self.TEST_CLUSTER)

        self.assertIsNotNone(obj.id)
        self.assertIsNotNone(obj)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from openstack.tests.unit import base

from otcextensions import sdk


class TestCase(base.TestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        sdk.load(self.cloud)
        self.cloud.config.config['rdsv3_api_version'] = '3'
        self.cloud.config.config['ddsv3_api_version'] = '3'

    def get_keystone_v3_token(
        self,
        project_name='admin',
    ):
        ets_rds = self.os_fixture._get_endpoint_templates('rdsv3')
        svc_rds = self.os_fixture.v3_token.add_service('rdsv3', name='rdsv3')
        svc_rds.add_standard_endpoints(region='RegionOne', **ets_rds)

        ets_cce = self.os_fixture._get_endpoint_templates('ccev2.0')
        svc_cce = self.os_fixture.v3_token.add_service(
            'ccev2.0', name='ccev2.0'
        )
        svc_cce.add_standard_endpoints(region='RegionOne', **ets_cce)

        ets_dds = self.os_fixture._get_endpoint_templates('ddsv3')
        svc_dds = self.os_fixture.v3_token.add_service('ddsv3', name='ddsv3')
        svc_dds.add_standard_endpoints(region='RegionOne', **ets_dds)

        return super(TestCase, self).get_keystone_v3_token()

    def get_rds_url(
        self,
        resource=None,
        append=None,
        base_url_append=None,
        qs_elements=None,
    ):
        url = self.get_mock_url(
            'rdsv3',
            resource=resource,
            append=append,
            base_url_append=base_url_append,
            qs_elements=qs_elements,
        )

        url = url % {'project_id': self.cloud.current_project_id}

        return url

    def get_cce_url(
        self,
        resource=None,
        append=None,
        base_url_append=None,
        qs_elements=None,
    ):
        endpoint_url = (
            'https://ccev2.0.example.com/' 'api/v3/projects/%(project_id)s'
        ) % {'project_id': self.cloud.current_project_id}
        # Strip trailing slashes, so as not to produce double-slashes below
        if endpoint_url.endswith('/'):
            endpoint_url = endpoint_url[:-1]
        to_join = [endpoint_url]
        qs = ''
        if base_url_append:
            to_join.append(base_url_append)
        if resource:
            to_join.append(resource)
        to_join.extend(append or [])
        if qs_elements is not None:
            qs = '?%s' % '&'.join(qs_elements)
        return '%(uri)s%(qs)s' % {'uri': '/'.join(to_join), 'qs': qs}

    def get_dds_url(
        self,
        resource=None,
        append=None,
        base_url_append=None,
        qs_elements=None,
    ):
        url = self.get_mock_url(
            'ddsv3',
            resource=resource,
            append=append,
            base_url_append=base_url_append,
            qs_elements=qs_elements,
        )

        url = url % {'project_id': self.cloud.current_project_id}

        return url



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cts.v1 import trace
from otcextensions.tests.unit.osclient.cts.v1 import fakes


class TestListTrace(fakes.TestCTS):

    objects = fakes.FakeTrace.create_multiple(3)

    columns = (
        'id', 'name', 'user', 'service_type', 'type',
        'resource_type', 'resource_name', 'resource_id',
        'source_ip', 'level', 'time'
    )

    columns_long = (
        'id', 'name', 'type', 'user', 'service_type',
        'resource_type', 'resource_name', 'resource_id',
        'source_ip', 'level', 'time', 'request', 'response'
    )

    data = []
    data_long = []

    for s in objects:
        data.append((
            s.id,
            s.name,
            s.user,
            s.service_type,
            s.type,
            s.resource_type,
            s.resource_name,
            s.resource_id,
            s.source_ip,
            s.level,
            s.time
        ))
        data_long.append((
            s.id,
            s.name,
            s.type,
            s.user,
            s.service_type,
            s.resource_type,
            s.resource_name,
            s.resource_id,
            s.source_ip,
            s.level,
            s.time,
            s.request,
            s.response
        ))

    def setUp(self):
        super(TestListTrace, self).setUp()

        self.cmd = trace.ListTrace(self.app, None)

        self.client.traces = mock.Mock()

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.traces.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.traces.assert_called_once_with(
            tracker='system'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_long(self):
        arglist = [
            '--long'
        ]

        verifylist = [
            ('long', True)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.traces.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.traces.assert_called_once_with(
            tracker='system'
        )

        self.assertEqual(self.columns_long, columns)
        self.assertEqual(self.data_long, list(data))

    def test_list_query(self):
        arglist = [
            '--tracker', 'trck',
            '--limit', '1',
            '--next', '2',
            '--service_type', '3',
            '--resource_type', '4',
            '--resource_id', '5',
            '--resource_name', '6',
            '--trace_name', '7',
            '--trace_id', '8',
            '--level', 'NORMAL',
            '--user', '9',
            '--start_time', '1970-01-01T00:00:00',
            '--end_time', '1970-01-01T00:00:00',
        ]

        verifylist = [
            ('tracker', 'trck'),
            ('limit', 1),
            ('next', 2),
            ('service_type', '3'),
            ('resource_type', '4'),
            ('resource_id', '5'),
            ('resource_name', '6'),
            ('trace_name', '7'),
            ('trace_id', '8'),
            ('level', 'NORMAL'),
            ('user', '9'),
            ('start_time', '1970-01-01T00:00:00'),
            ('end_time', '1970-01-01T00:00:00'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.traces.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.traces.assert_called_once_with(
            level='NORMAL',
            limit=1,
            next=2,
            res_id='5',
            res_name='6',
            res_type='4',
            service_type='3',
            to=0,
            trace_id='8',
            trace_name='7',
            tracker='trck',
            user='9',
            **{
                'from': 0,
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.dcaas.v2 import virtual_gateway as _virtual_gateway
from otcextensions.sdk.dcaas.v2 import connection as _connection
from otcextensions.sdk.dcaas.v2 import virtual_interface as _virtual_interface
from otcextensions.sdk.dcaas.v2 import endpoint_group as _endpoint_group


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Virtual gateways ========
    def virtual_gateways(self, **query):
        """Retrieve a generator of virtual gateways

        :returns: A generator of virtual gateways
         :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.VirtualGateway`
         instances
        """
        return self._list(_virtual_gateway.VirtualGateway, **query)

    def create_virtual_gateway(self, **attrs):
        """Create a new virtual gateway from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.VirtualGateway`,
            comprised of the properties on the VirtualGateway class.
        :returns: The results of virtual gateway creation
        :rtype: :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.
            VirtualGateway`
        """
        return self._create(_virtual_gateway.VirtualGateway,
                            prepend_key=False, **attrs)

    def get_virtual_gateway(self, virtual_gateway):
        """Get a virtual_gateway

        :param virtual_gateway: The value can be the ID of a virtual_gateway
             or a :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.
             VirtualGateway` instance.
        :returns: Virtual gateway instance
        :rtype: :class:`~otcextensions.sdk.dcaasdcaas.v2.virtual_gateway.
            VirtualGateway`
        """
        return self._get(_virtual_gateway.VirtualGateway, virtual_gateway)

    def delete_virtual_gateway(self, virtual_gateway, ignore_missing=True):
        """Delete a virtual_gateway

        :param virtual_gateway: The value can be the ID of a virtual gateway
             or a :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.
             VirtualGateway` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the virtual gateway does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: `None`
        """
        return self._delete(_virtual_gateway.VirtualGateway, virtual_gateway,
                            ignore_missing=ignore_missing)

    def update_virtual_gateway(self, virtual_gateway, **attrs):
        """Update virtual gateway attributes

        :param virtual_gateway: The id or an instance of
            :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.VirtualGateway`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.VirtualGateway`

        :rtype: :class:`~otcextensions.sdk.dcaas.v2.virtual_gateway.
            VirtualGateway`
        """
        return self._update(_virtual_gateway.VirtualGateway,
                            virtual_gateway, **attrs)

    def find_virtual_gateway(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single virtual gateway

        :param name_or_id: The name or ID of a virtual gateway
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the zone does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent virtual gateway.

        :returns: ``None``
        """
        return self._find(_virtual_gateway.VirtualGateway, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    # ======== Connections ========
    def connections(self, **query):
        """Retrieve a generator of connections

        :returns: A generator of connections
            :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
            instances
        """
        return self._list(_connection.Connection, **query)

    def create_connection(self, **attrs):
        """Create a new connection from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`,
            comprised of the properties on the Connection class.
        :returns: The results of connection creation
        :rtype: :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
        """
        return self._create(_connection.Connection, prepend_key=False, **attrs)

    def get_connection(self, connection):
        """Get a connection

        :param connection: The value can be the ID of a connection
             or a :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
             instance.
        :returns: Connection instance
        :rtype: :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
        """
        return self._get(_connection.Connection, connection)

    def delete_connection(self, connection, ignore_missing=True):
        """Delete a connection

        :param connection: The value can be the ID of a connection
             or a :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the connection does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent connection.

        :returns: `None`
        """
        return self._delete(_connection.Connection, connection,
                            ignore_missing=ignore_missing)

    def update_connection(self, connection, **attrs):
        """Update connection attributes

        :param connection: The id or an instance of
            :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`

        :rtype:
            :class:`~otcextensions.sdk.dcaas.v2.connection.Connection`
        """
        return self._update(_connection.Connection, connection, **attrs)

    def find_connection(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single connection

        :param name_or_id: The name or ID of a connection
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the connection does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent connection.

        :returns: ``None``
        """
        return self._find(_connection.Connection, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    # ======== Virtual interface ========
    def virtual_interfaces(self, **query):
        """Retrieve a generator of virtual interfaces

        :returns: A generator of virtual interfaces
            :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface` instances
        """
        return self._list(_virtual_interface.VirtualInterface, **query)

    def create_virtual_interface(self, **attrs):
        """Create a new virtual interface from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`, comprised of the properties on the Connection
            class.
        :returns: The results of virtual interface creation
        :rtype: :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`
        """
        return self._create(_virtual_interface.VirtualInterface,
                            prepend_key=False, **attrs)

    def get_virtual_interface(self, virtual_interface):
        """Get a virtual_interface

        :param virtual_interface: The value can be the ID of a
            virtual_interface or a :class:`~otcextensions.sdk.dcaas.v2.
            virtual_interface.VirtualInterface` instance.
        :returns: Virtual interface instance
        :rtype:
            :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`
        """
        return self._get(_virtual_interface.VirtualInterface,
                         virtual_interface)

    def delete_virtual_interface(self, virtual_interface, ignore_missing=True):
        """Delete a virtual interface

        :param virtual_interface: The value can be the ID of a virtual
            interface or a :class:`~otcextensions.sdk.dcaas.v2.
            virtual_interface.VirtualInterface` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the virtual interface does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent virtual interface.

        :returns: `None`
        """
        return self._delete(_virtual_interface.VirtualInterface,
                            virtual_interface, ignore_missing=ignore_missing)

    def update_virtual_interface(self, virtual_interface, **attrs):
        """Update virtual interface attributes

        :param virtual_interface: The id or an instance of
            :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`

        :rtype: :class:`~otcextensions.sdk.dcaas.v2.virtual_interface.
            VirtualInterface`
        """
        return self._update(_virtual_interface.VirtualInterface,
                            virtual_interface, **attrs)

    def find_virtual_interface(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single virtual interface

        :param name_or_id: The name or ID of a virtual interface
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the virtual interface does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent virtual interface.

        :returns: ``None``
        """
        return self._find(_virtual_interface.VirtualInterface, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    # ======== Direct Connect Endpoint Group ========
    def endpoint_groups(self, **query):
        """Retrieve a generator of direct connect endpoint groups.

        :returns: A generator of direct connect endpoint groups.
            :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
                DirectConnectEndpointGroup` instances
        """
        return self._list(_endpoint_group.DirectConnectEndpointGroup, **query)

    def create_endpoint_group(self, **attrs):
        """Create a new direct connect endpoint group from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`, comprised of the properties on the
            DirectConnectEndpointGroup class.
        :returns: The results of endpoint group creation
        :rtype: :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`
        """
        return self._create(_endpoint_group.DirectConnectEndpointGroup,
                            prepend_key=False, **attrs)

    def get_endpoint_group(self, endpoint_group):
        """Get a direct connect endpoint group.

        :param endpoint_group: The value can be the ID of a
            endpoint group or a :class:`~otcextensions.sdk.dcaas.v2.
            endpoint_group.DirectConnectEndpointGroup` instance.
        :returns: Endpoint Group instance
        :rtype:
            :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`
        """
        return self._get(_endpoint_group.DirectConnectEndpointGroup,
                         endpoint_group)

    def delete_endpoint_group(self, endpoint_group, ignore_missing=True):
        """Delete a Direct Connect Endpoint Group.

        :param endpoint_group: The value can be the ID of a endpoint
            group or a :class:`~otcextensions.sdk.dcaas.v2.
            endpoint_group.DirectConnectEndpointGroup` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the endpoint group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent endpoint group.

        :returns: `None`
        """
        return self._delete(_endpoint_group.DirectConnectEndpointGroup,
                            endpoint_group, ignore_missing=ignore_missing)

    def update_endpoint_group(self, endpoint_group, **attrs):
        """Update Direct Connect Endpoint Group attributes

        :param endpoint_group: The id or an instance of
            :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`

        :rtype: :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`
        """
        return self._update(_endpoint_group.DirectConnectEndpointGroup,
                            endpoint_group, **attrs)

    def find_endpoint_group(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single Endpoint Group.

        :param name_or_id: The name or ID of a endpoint group.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the endpoint group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent endpoint group.

        :returns: one object of class
            :class:`~otcextensions.sdk.dcaas.v2.endpoint_group.
            DirectConnectEndpointGroup`

        """
        return self._find(_endpoint_group.DirectConnectEndpointGroup,
                          name_or_id, ignore_missing=ignore_missing,
                          **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import signature_binding

EXAMPLE_BIND = {
    "sign_id": "0b0e8f456b8742218af75f945307173c",
    "publish_ids": ["40e7162dc6b94bbbbb1a60d2a24b1b0c"]
}


class TestSignatureBind(base.TestCase):

    def test_basic(self):
        sot = signature_binding.SignatureBind()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/sign-bindings',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('bindings', sot.resources_key)

    def test_make_it(self):
        sot = signature_binding.SignatureBind(**EXAMPLE_BIND)
        self.assertEqual(EXAMPLE_BIND['sign_id'], sot.sign_id)
        self.assertEqual(EXAMPLE_BIND['publish_ids'], sot.publish_ids)


EXAMPLE_NOTBOUND = {
    "run_env_name": "RELEASE",
    "group_name": "api_group_001",
    "remark": "Mock backend API",
    "publish_id": "9f27d1dc4f4242a9abf88e563dbfc33d",
    "group_id": "c77f5e81d9cb4424bf704ef2b0ac7600",
    "name": "Api_mock",
    "run_env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "id": "3a955b791bd24b1c9cd94c745f8d1aad",
    "type": 1,
    "auth_type": "IAM",
    "req_uri": "/test/mock"
}


class TestNotBoundApi(base.TestCase):

    def test_basic(self):
        sot = signature_binding.NotBoundApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/sign-bindings/unbinded-apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = signature_binding.NotBoundApi(**EXAMPLE_NOTBOUND)
        self.assertEqual(EXAMPLE_NOTBOUND['run_env_name'], sot.run_env_name)
        self.assertEqual(EXAMPLE_NOTBOUND['group_name'], sot.group_name)
        self.assertEqual(EXAMPLE_NOTBOUND['remark'], sot.remark)
        self.assertEqual(EXAMPLE_NOTBOUND['publish_id'], sot.publish_id)
        self.assertEqual(EXAMPLE_NOTBOUND['group_id'], sot.group_id)


EXAMPLE_BOUND = {
    "api_id": "5f918d104dc84480a75166ba99efff21",
    "group_name": "api_group_001",
    "binding_time": "2020-08-03T04:00:11Z",
    "env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
    "env_name": "RELEASE",
    "sign_name": "signature_demo",
    "api_type": 1,
    "api_name": "Api_http",
    "id": "25082bd52f74442bb1d273993d567938",
    "api_remark": "Web backend API",
    "publish_id": "40e7162dc6b94bbbbb1a60d2a24b1b0c"
}


class TestBoundApi(base.TestCase):

    def test_basic(self):
        sot = signature_binding.BoundApi()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/sign-bindings/binded-apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('bindings', sot.resources_key)

    def test_make_it(self):
        sot = signature_binding.BoundApi(**EXAMPLE_BOUND)
        self.assertEqual(EXAMPLE_BOUND['api_id'], sot.api_id)
        self.assertEqual(EXAMPLE_BOUND['group_name'], sot.group_name)
        self.assertEqual(EXAMPLE_BOUND['binding_time'], sot.bind_at)
        self.assertEqual(EXAMPLE_BOUND['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_BOUND['env_name'], sot.env_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from urllib.parse import urlunparse
from urllib.parse import urlparse

from openstack import exceptions
from openstack import utils

from openstack.compute.v2 import server


class Server(server.Server):

    def _prepare_override_url(self, url, new_version):
        parsed = urlparse(url)
        path_parts = parsed.path.split('/')
        if len(path_parts) >= 2:
            path_parts[1] = new_version
        new_path = '/'.join(path_parts)
        new_parsed_url = parsed._replace(path=new_path)
        return urlunparse(new_parsed_url)

    def _get_tag_struct(self, tag):
        tag_pairs = tag.split('=')
        if len(tag_pairs) == 2:
            (k, v) = (tag_pairs[0], tag_pairs[1])
        else:
            (k, v) = (tag, "")
        return {
            'key': k,
            'value': v
        }

    def add_tag(self, session, tag):
        """Adds a single tag to the resource.

        :param session: The session to use for making this request.
        :param tag: The tag as a string.
        """
        session.endpoint_override = self._prepare_override_url(
            session.get_endpoint(),
            'v1'
        )
        url = utils.urljoin('cloudservers', self.id,
                            'tags', 'action')
        body = {
            'action': 'create',
            'tags': [self._get_tag_struct(tag)]
        }
        response = session.post(url, json=body)
        exceptions.raise_from_response(response)
        # we do not want to update tags directly
        tags = self.tags
        tags.append(tag)
        self._body.attributes.update({
            'tags': list(set(tags))
        })
        session.endpoint_override = None
        return self

    def remove_tag(self, session, tag):
        """Removes a single tag from the specified server.

        :param session: The session to use for making this request.
        :param tag: The tag as a string.
        """
        session.endpoint_override = self._prepare_override_url(
            session.get_endpoint(),
            'v1'
        )
        url = utils.urljoin('cloudservers', self.id,
                            'tags', 'action')
        body = {
            'action': 'delete',
            'tags': [self._get_tag_struct(tag)]
        }
        response = session.post(url, json=body)
        exceptions.raise_from_response(response)
        # we do not want to update tags directly
        tags = self.tags
        try:
            # NOTE(gtema): if tags were not fetched, but request suceeded
            # it is ok. Just ensure tag does not exist locally
            tags.remove(tag)
        except ValueError:
            pass  # do nothing!
        self._body.attributes.update({
            'tags': tags
        })
        session.endpoint_override = None
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class RestoreRecord(resource.Resource):

    resources_key = 'restore_record_response'

    base_path = '/instances/%(instance_id)s/restores'

    # capabilities
    allow_list = True
    allow_create = True

    _query_mapping = resource.QueryParameters(
        'limit', 'start', 'begin_time', 'end_time',
        begin_time='beginTime',
        end_time='endTime')

    # Properties
    #: Instance ID
    instance_id = resource.URI('instance_id')
    #: Backup description
    backup_description = resource.Body('backup_remark')
    #: Backup ID
    backup_id = resource.Body('backup_id')
    #: Backup name
    backup_name = resource.Body('backup_name')
    #: Time at which the backup task is created.
    created_at = resource.Body('created_at')
    #: Restore description
    description = resource.Body('remark')
    #: Error code returned if DCS instance restore fails.
    error_code = resource.Body('error_code')
    #: Restore ID
    id = resource.Body('restore_id', alternate_id=True)
    #: Restore description
    restore_description = resource.Body('restore_remark', alias='description')
    #: Restore name
    restore_name = resource.Body('restore_name')
    #: Restore progress.
    progress = resource.Body('progress')
    #: Restore status.
    #: * waiting: DCS instance restore is waiting to begin.
    #: * restoring: DCS instance restore is in progress.
    #: * succeed: DCS instance restore succeeded.
    #: * failed: DCS instance restore failed.
    status = resource.Body('status')
    #: Time at which DCS instance restore is completed.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import ast
import typing as ty

from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.lts.v2 import stream as _stream


class Group(resource.Resource):
    resource_key = 'log_groups'
    resources_key = 'log_groups'
    base_path = '/groups'

    # capabilities
    allow_create = True
    allow_fetch = False
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
    )

    # Properties
    #: Time when a log group was created
    creation_time = resource.Body('creation_time')
    #: ID of the log group.
    id = resource.Body('log_group_id', alternate_id=True)
    #: Name of the log group.
    name = resource.Body('log_group_name')
    #: Log retention duration, in days (fixed to 7 days).
    ttl_in_days = resource.Body('ttl_in_days', type=int)
    #: Log group tag.
    tag = resource.Body('tag')

    def _prepare_request_body(
        self,
        patch,
        prepend_key,
        *,
        resource_request_key=None,
    ):
        body: ty.Union[ty.Dict[str, ty.Any], ty.List[ty.Any]]
        if not self._store_unknown_attrs_as_properties:
            # Default case
            body = self._body.dirty
        else:
            body = self._unpack_properties_to_resource_root(
                self._body.dirty
            )
        return body

    def commit(
        self,
        session,
        prepend_key=True,
        has_body=True,
        retry_on_conflict=None,
        base_path=None,
        *,
        microversion=None,
        **kwargs,
    ):
        # The id cannot be dirty for an commit
        self._body._dirty.discard("id")

        # Only try to update if we actually have anything to commit.
        if not self.requires_commit:
            return self

        # Avoid providing patch unconditionally to avoid breaking subclasses
        # without it.
        if self.commit_jsonpatch:
            kwargs['patch'] = True

        request = self._prepare_request(
            prepend_key=prepend_key,
            base_path=base_path,
            **kwargs,
        )
        if microversion is None:
            microversion = self._get_microversion(session)

        return self._commit(
            session,
            request,
            "POST",
            microversion,
            has_body=has_body,
            retry_on_conflict=retry_on_conflict,
        )

    def create_stream(self, session, query):
        """Method to create a stream in log group

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param dict query: Additional parameters to create stream
        """
        url = utils.urljoin(self.base_path, self.id, '/streams')
        resp = session.post(url, json=query)
        stream = _stream.Stream()
        stream._translate_response(resp)
        return stream

    def delete_stream(self, session, log_stream_id, ignore_missing):
        """Method to delete stream from log group

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str log_stream_id: Id of the stream to be deleted
        :param bool ignore_missing: Should it be deleted
         if doesn't exist or not
        """
        url = utils.urljoin(self.base_path, self.id, '/streams/',
                            log_stream_id)
        resp = session.delete(url)
        if resp.status_code == 404:
            if not ignore_missing:
                raise exceptions.NotFoundException(
                    str(ast.literal_eval(resp._content.decode('utf-8'))))
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class DatasetSync(resource.Resource):
    base_path = "/datasets/%(uri_dataset_id)s/sync-data"

    allow_create = True
    allow_fetch = True

    # Properties
    #: Number of added samples.
    add_sample_count = resource.Body("add_sample_count", type=int)
    #: Task creation time.
    create_time = resource.Body("create_time", type=int)
    #: Resource URI Dataset ID.
    uri_dataset_id = resource.URI("uri_dataset_id")
    #: Dataset ID.
    dataset_id = resource.Body("dataset_id")
    #: Number of deleted samples.
    deleted_sample_count = resource.Body("deleted_sample_count", type=int)
    #: Task running time.
    duration_time = resource.Body("duration_time", type=int)
    #: Error code of a failed API call. For details, see Error Code.
    #:  This parameter is not included when the API call succeeds.
    error_code = resource.Body("error_code", type=str)
    #: Error message of a failed API call.
    #:  This parameter is not included when the API call succeeds.
    error_msg = resource.Body("error_msg", type=str)
    #: Status of a data synchronization task. Possible values are as follows:
    #: \nQUEUING
    #: \nSTARTING
    #: \nRUNNING
    #: \nFAILED
    #: \nCOMPLETED
    #: \nNOT_EXIST
    status = resource.Body("status")
    #: Synchronization task ID.
    task_id = resource.Body("task_id")
    #: Total number of samples.
    total_sample_count = resource.Body("total_sample_count", type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import exceptions


class OperationSpec(resource.Resource):
    service_type = resource.Body('service_type')
    resource_type = resource.Body('resource_type')
    trace_names = resource.Body('trace_names', type=list)


class UserSpec(resource.Resource):
    user_group = resource.Body('user_group')
    user_list = resource.Body('user_list', type=list)


class FilterSpec(resource.Resource):
    condition = resource.Body('condition')
    is_support_filter = resource.Body('is_support_filter', type=bool)
    rule = resource.Body('rule', type=list)


class KeyEvent(resource.Resource):
    base_path = '/notifications'
    resources_key = 'notifications'
    allow_list = True
    allow_create = True
    allow_delete = True
    allow_commit = True
    requires_id = False

    _query_mapping = resource.QueryParameters('notification_name')
    notification_name = resource.Body('notification_name')
    operation_type = resource.Body('operation_type')
    operations = resource.Body('operations', type=list, list_type=dict)
    notify_user_list = resource.Body('notify_user_list', list_type=list)
    topic_id = resource.Body('topic_id')
    filter = resource.Body('filter', type=FilterSpec)
    status = resource.Body('status')
    notification_id = resource.Body('notification_id')
    notification_type = resource.Body('notification_type')
    project_id = resource.Body('project_id')
    create_time = resource.Body('create_time')

    def delete_key(self, session):
        path = f'{self.base_path}?notification_id={self.notification_id}'
        response = session.delete(path)
        exceptions.raise_from_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.anti_ddos.v1 import config

EXAMPLE = {
    "traffic_limited_list": [
        {
            "traffic_pos_id": 1,
            "traffic_per_second": 10,
            "packet_per_second": 2000
        }
    ],
    "http_limited_list": [
        {
            "http_request_pos_id": 1,
            "http_packet_per_second": 100
        }
    ],
    "connection_limited_list": [
        {
            "cleaning_access_pos_id": 1,
            "new_connection_limited": 10,
            "total_connection_limited": 30
        }
    ],
    "extend_ddos_config": [
        {
            "new_connection_limited": 80,
            "total_connection_limited": 700,
            "http_packet_per_second": 500000,
            "traffic_per_second": 1000,
            "packet_per_second": 200000,
            "setID": 33
        },
    ]

}


class TestConfig(base.TestCase):

    def setUp(self):
        super(TestConfig, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = config.Config()

        self.assertEqual('/antiddos/query_config_list', sot.base_path)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = config.Config(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['traffic_limited_list'],
            sot.traffic_limited_list)
        self.assertEqual(
            EXAMPLE['http_limited_list'],
            sot.http_limited_list)
        self.assertEqual(
            EXAMPLE['connection_limited_list'],
            sot.connection_limited_list)
        self.assertEqual(
            EXAMPLE['extend_ddos_config'],
            sot.extend_ddos_config)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.anti_ddos.v1 import floating_ip

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "enable_L7": False,
    "traffic_pos_id": 2,
    "http_request_pos_id": 1,
    "cleaning_access_pos_id": 1,
    "app_type_id": 1,
    "floating_ip_id": FAKE_ID
}


class TestFloatingIP(base.TestCase):

    def setUp(self):
        super(TestFloatingIP, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = floating_ip.FloatingIP()

        self.assertEqual('/antiddos', sot.base_path)
        self.assertEqual('ddosStatus', sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = floating_ip.FloatingIP(**EXAMPLE)
        self.assertEqual(EXAMPLE['floating_ip_id'], sot.id)
        self.assertEqual(EXAMPLE['enable_L7'], sot.is_enable_l7)
        self.assertEqual(EXAMPLE['traffic_pos_id'], sot.traffic_pos_id)
        self.assertEqual(
            EXAMPLE['http_request_pos_id'],
            sot.http_request_pos_id)
        self.assertEqual(
            EXAMPLE['cleaning_access_pos_id'],
            sot.cleaning_access_pos_id)
        self.assertEqual(
            EXAMPLE['app_type_id'],
            sot.app_type_id)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Delete dependency version
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

u = 'https://fg-test-files.obs.eu-de.otc.t-systems.com/dependency.zip'
attrs = {
    'depend_link': u,
    'depend_type': 'obs',
    'runtime': 'Python3.10',
    'name': 'test-dep-1'
}
dv = conn.functiongraph.create_dependency_version(**attrs)
conn.functiongraph.delete_dependency_version(dv)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid

from otcextensions.tests.functional.sdk.dns import TestDns

_logger = openstack._log.setup_logging('openstack')


class TestFloatingIps(TestDns):
    uuid_v4 = uuid.uuid4().hex[:8]
    ptr_dname = uuid_v4 + 'dns.ptr.test.com'

    def setUp(self):
        super(TestFloatingIps, self).setUp()
        pub_net = self.conn.get_network('admin_external_net')
        self.floating_ip = self.conn.network.create_ip(
            floating_network_id=pub_net.id
        )
        self.floatingip = self.client.set_floating_ip(
            floating_ip=('eu-de:' + self.floating_ip.id),
            ptrdname=self.ptr_dname
        )

    def tearDown(self):
        self.client.unset_floating_ip(
            floating_ip=self.floatingip,
        )
        self.conn.network.delete_ip(self.floating_ip)
        super(TestFloatingIps, self).tearDown()

    def test_list_floatingips(self):
        ips = self.client.floating_ips()
        self.assertIsNotNone(ips)

    def test_get_floatingip(self):
        ip = self.client.get_floating_ip(self.floatingip.id)
        self.assertEqual(ip['address'], self.floatingip['address'])

    # BUG
    # https://jira.tsi-dev.otc-service.com/browse/ONS-5716
    # def test_update_floatingip(self):
    #     ip = self.client.update_floating_ip(
    #         floating_ip=self.floatingip,
    #         ttl=500
    #     )
    #     self.assertEqual(ip['description'], 'updated')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts sample v2 action implementations"""

import logging

from cliff import columns as cliff_columns
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

SORT_ORDER_CHOICES = ["asc", "desc"]

SAMPLE_STATE_CHOICES = [
    "all",
    "none",
    "uncheck",
    "accepted",
    "rejected",
    "unreviewed",
    "reviewed",
    "workforce_sampled",
    "workforce_sampled_uncheck",
    "workforce_sampled_checked",
    "workforce_sampled_accepted",
    "workforce_sampled_rejected",
    "auto_annotation",
]


class SampleType(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "Image",
        1: "Text",
        2: "Speech",
        4: "Table",
        6: "Video",
        9: "Custom_Format",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class LabelType(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "image classification",
        1: "object detection",
        100: "text classification",
        101: "named entity recognition",
        102: "text triplet",
        200: "sound classification",
        201: "speech content",
        202: "speech paragraph labeling",
        400: "table dataset",
        600: "video labeling",
        900: "custom format",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class DataSourceType(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "OBS bucket (default value)",
        1: "GaussDB(DWS)",
        2: "DLI",
        3: "RDS",
        4: "MRS",
        5: "AI Gallery",
        6: "Inference service",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


DATASOURCE_KEYS_MAP = {
    "cluster_id": "ID of a MRS cluster",
    "cluster_mode": (
        "Running mode of a MRS cluster. Options are as follows:\n"
        "* 0: normal cluster\n"
        "* 1: security cluster"
    ),
    "cluster_name": "Name of a MRS cluster",
    "database_name": (
        "Name of the database to which the table dataset is imported."
    ),
    "input": "HDFS path of a table dataset",
    "ip": "IP address of your GaussDB(DWS) cluster",
    "port": "Port number of your GaussDB(DWS) cluster",
    "queue_name": "DLI queue name of a table dataset",
    "subnet_id": "Subnet ID of a MRS cluster",
    "table_name": "Name of the table to which a table dataset is imported",
    "user_name": "Username which is mandatary for GaussDB(DWS)",
    "user_password": "User password which is mandatary for GaussDB(DWS)",
    "vpc_id": "ID of the vpc of a MRS cluster",
}

LABEL_KEYS_MAP = {
    "annotated_by": (
        "Video labeling method, which is used to distinguish whether "
        "a video is labeled manually or automatically."
        "The options are as follows:\n"
        "* human: manual labeling\n"
        "* auto: automatic labeling"
    ),
    "id": "Label ID",
    "name": "Label name",
    "score": "Confidence",
    "type": (
        "Label type. The options are as follows:\n"
        "* 0: image classification\n"
        "* 1: object detection\n"
        "* 100: text classification\n"
        "* 101: named entity recognition\n"
        "* 102: text triplet relationship\n"
        "* 103: text triplet entity"
        "* 200: speech classification"
        "* 201: speech content"
        "* 202: speech paragraph labeling"
        "* 600: video classification"
    ),
}


_formatters = {
    "labels": cli_utils.YamlFormat,
    "sample_time": cli_utils.UnixTimestampFormatter,
    "source": cli_utils.WrapText,
    "metadata": cli_utils.YamlFormat,
    "sample_type": SampleType,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        "location",
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class AddDatasetSamples(command.ShowOne):
    _description = _("Add samples to a dataset.")

    columns = ("success", "results")
    column_headers = ("Success", "Results")

    def get_parser(self, prog_name):
        parser = super(AddDatasetSamples, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "--file-path",
            metavar="<file_path>",
            help=_("Sample file path to upload."),
        )
        parser.add_argument(
            "--directory-path",
            metavar="<directory_path>",
            help=_("Samples Directory path."),
        )
        # parser.add_argument(
        #     "--name",
        #     required=True,
        #     metavar="<name>",
        #     help=_("Name of sample file."),
        # )
        # parser.add_argument(
        #     "--data",
        #     metavar="<data>",
        #     required=True,
        #     help=_("Byte data of sample file."),
        # )
        parser.add_argument(
            "--encoding",
            metavar="{UTF-8, GBK, GB2312}",
            type=lambda s: s.upper(),
            choices=["UTF-8", "GBK", "GB2312"],
            help=_(
                "Encoding type of sample files, which is used to upload "
                ".txt or .csv files.\nThe default value is UTF-8."
            ),
        )
        parser.add_argument(
            "--metadata",
            metavar="<key=value>",
            dest="metadata",
            action=parseractions.KeyValueAction,
            help=_(
                "Key-value pair of the sample metadata attribute.\n"
                "Repeat the option to set multiple metadata attributes.\n"
                "Example:\n--metadata @modelarts:hard=0\n"
                "--metadata @modelarts:hard_coefficient=0.22\n"
                "--metadata @modelarts:hard_reasons=[1,2]\n"
                "--metadata @modelarts:size=[100,200,3]"
            ),
        )

        parser.add_argument(
            "--sample-type",
            choices=list(SampleType.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Sample type. The options are as follows:\n" + SampleType.STR
            ),
        )
        parser.add_argument(
            "--data-source-path",
            metavar="<data_source_path>",
            help=_("Data source path."),
        )
        parser.add_argument(
            "--data-source-type",
            choices=list(DataSourceType.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Data source type. The options are as follows:\n"
                + DataSourceType.STR
            ),
        )
        parser.add_argument(
            "--data-source-info",
            action=parseractions.MultiKeyValueAction,
            metavar="<key1=value1,key2=value2,..>",
            dest="data_source_info",
            required_keys=[],
            optional_keys=list(DATASOURCE_KEYS_MAP.keys()),
            help=_(
                "Information required for importing a table data source.\n"
                "Supported keys:\n"
                + "\n".join(
                    [
                        f"{key}: {value}"
                        for key, value in DATASOURCE_KEYS_MAP.items()
                    ]
                )
                + "\nExample:\n"
                "--data-source-info cluster_id=1234,cluster_mode=1"
            ),
        )
        parser.add_argument(
            "--data-with-column-header",
            action="store_true",
            help=_("Whether the first row in the data file is a column name."),
        )
        parser.add_argument(
            "--schema-map",
            metavar="src_name=<src_name>,dest_name=<dest_name>",
            required_keys=["src_name", "dest_name"],
            optional_keys=[],
            action=parseractions.MultiKeyValueAction,
            help=_(
                "Schema mapping information corresponding to the table data.\n"
                "src_name: Name of the source column.\n"
                "dest_name: Name of the destination column.\n"
                "Example:\n"
                "--schema-map src_name=colnameA,dest_name=colnameB\n"
                "--schema-map src_name=colnameX,dest_name=colnameY"
            ),
        )
        parser.add_argument(
            "--label",
            metavar="key1=<value1>,key2=<value2>,..",
            action=parseractions.MultiKeyValueAction,
            dest="labels",
            required_keys=[],
            optional_keys=list(LABEL_KEYS_MAP.keys()),
            help=_(
                "Sample label list.\n"
                "Supported keys:\n"
                + "\n".join(
                    [
                        f"{key}: {value}"
                        for key, value in LABEL_KEYS_MAP.items()
                    ]
                )
                + "\nExample:\n"
                "--label cluster_id=1234,cluster_mode=1"
            ),
        )
        parser.add_argument(
            "--to-be-confirmed",
            action="store_true",
            help=_(
                "Whether to import labels to the to-be-confirmed dataset.\n"
                "Currently, to-be-confirmed datasets only support categories "
                "of image classification and object detection."
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        sample = {}
        for arg in (
            # "name",
            # "data",
            "file_path",
            "directory_path",
            "encoding",
            "sample_type",
            "metadata",
            "labels",
        ):
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                sample[arg] = val

        data_source = {}
        if parsed_args.data_source_path:
            data_source["data_path"] = parsed_args.data_source_path
        if parsed_args.data_source_type:
            data_source["data_type"] = parsed_args.data_source_type
        if parsed_args.schema_map:
            data_source["schema_map"] = parsed_args.schema_map
        data_source_info = parsed_args.data_source_info
        if data_source_info:
            if len(data_source_info) > 1:
                msg = "ERROR: --data-source-info argument cannot be repeated"
                raise exceptions.CommandError(msg)
            data_source["source_info"] = data_source_info[0]
        if parsed_args.data_with_column_header:
            data_source["with_column_header"] = True
        if data_source:
            sample.update(data_source=data_source)

        data = client.add_dataset_samples(dataset.id, **sample)

        formatters = {
            "results": cli_utils.YamlFormat,
        }
        return (
            self.column_headers,
            utils.get_item_properties(
                data, self.columns, formatters=formatters
            ),
        )


class ListDatasetSamples(command.Lister):
    _description = _("Get details about a Sample.")
    columns = (
        "Id",
        "Sample Type",
        "Sample Status",
        "Sample Time",
    )

    def get_parser(self, prog_name):
        parser = super(ListDatasetSamples, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "--email",
            metavar="<email>",
            help=_("Email address of a labeling team member."),
        )
        parser.add_argument(
            "--high-score",
            metavar="<high_score>",
            help=_("Upper confidence limit. The default value is 1."),
        )
        parser.add_argument(
            "--label-name",
            metavar="<label_name>",
            help=_("Label name."),
        )
        parser.add_argument(
            "--label-type",
            choices=list(LabelType.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Labeling type. The options are as follows:" + LabelType.STR
            ),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each "
                "page. The value ranges from 1 to 100. "
                "The default value is 10."
            ),
        )
        parser.add_argument(
            "--locale",
            metavar="<locale>",
            help=_(
                "Language. The options are as follows:"
                "\nen-us: English (default value)."
            ),
        )
        parser.add_argument(
            "--low-score",
            metavar="<low_score>",
            help=_("Lower confidence limit. The default value is 0."),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start page of the paging list. The default value is 0."),
        )
        parser.add_argument(
            "--order",
            metavar="{asc, desc}",
            type=lambda s: s.lower(),
            choices=["asc", "desc"],
            help=_("Sorting order. Default value: desc"),
        )
        parser.add_argument(
            "--preview",
            action="store_true",
            help=_("Whether to support preview."),
        )
        parser.add_argument(
            "--process-parameter",
            metavar="<process_parameter>",
            help=_(
                "Image resizing setting, which is the same as the "
                "OBS resizing setting."
            ),
        )
        parser.add_argument(
            "--sample-state",
            metavar="{" + ",".join(SAMPLE_STATE_CHOICES) + "}",
            type=lambda s: s.lower(),
            choices=SAMPLE_STATE_CHOICES,
            help=_("Sample status."),
        )
        parser.add_argument(
            "--sample-type",
            choices=list(SampleType.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Sample type. The options are as follows:" + SampleType.STR
            ),
        )
        parser.add_argument(
            "--search-conditions",
            metavar="<search_conditions>",
            help=_(
                "Multi-dimensional search condition after URL encoding. "
                "The relationship between multiple search "
                "conditions is AND."
            ),
        )
        parser.add_argument(
            "--version-id",
            metavar="<version_id>",
            help=_("Dataset version ID."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        attrs_list = (
            "email",
            "high_score",
            "label_name",
            "label_type",
            "limit",
            "locale",
            "low_score",
            "offset",
            "order",
            "preview",
            "process_parameter",
            "sample_state",
            "sample_type",
            "search_conditions",
            "version_id",
        )
        query_params = {}
        for arg in attrs_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        data = client.dataset_samples(dataset.id, **query_params)

        formatters = {
            "Sample Type": SampleType,
            "Sample Time": cli_utils.UnixTimestampFormatter,
        }

        return (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )


class ShowDatasetSample(command.ShowOne):
    _description = _("Show instance details")

    def get_parser(self, prog_name):
        parser = super(ShowDatasetSample, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "sampleId",
            metavar="<sampleId>",
            help=_("Dataset Sample ID."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2
        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        obj = client.get_dataset_sample(dataset.id, parsed_args.sampleId)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteDatasetSamples(command.ShowOne):
    _description = _(
        "This API is used to delete the samples of a dataset in batches."
    )

    def get_parser(self, prog_name):
        parser = super(DeleteDatasetSamples, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "sampleId",
            metavar="<sampleId>",
            nargs="+",
            help=_("ID of Dataset sample(s) to be deleted."),
        )
        parser.add_argument(
            "--delete-source",
            action="store_true",
            help=_("Whether to delete the source file. (default: false.)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2
        delete_source = False
        if parsed_args.delete_source:
            delete_source = parsed_args.delete_source

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        obj = client.delete_dataset_samples(
            dataset.id, parsed_args.sampleId, delete_source
        )
        formatters = {
            "results": cli_utils.YamlFormat,
        }

        hidden = ["samples", "delete_source", "location"]

        (
            display_columns,
            columns,
        ) = sdk_utils.get_osc_show_columns_for_sdk_resource(obj, {}, hidden)

        data = utils.get_item_properties(obj, columns, formatters=formatters)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk import sdk_resource
from otcextensions.sdk.cce.v1 import _base
from otcextensions.sdk.cce.v1 import cluster_node


class NodeListSpec(sdk_resource.Resource):
    # Properties
    host_list = resource.Body('hostList', type=list,
                              list_type=cluster_node.ClusterNode)


class ClusterNodeList(_base.Resource):
    # Properties
    #: Spec
    spec = resource.Body('spec', type=NodeListSpec)


class ClusterSpec(sdk_resource.Resource):

    # Properties
    #: Description
    #: *Type:str
    description = resource.Body('description')
    #: Availability zone
    #: *Type:str*
    availability_zone = resource.Body('az')
    #: CPU size
    #: *Type:int*
    cpu = resource.Body('cpu', type=int)
    #: Memory size
    #: *Type:int*
    memory = resource.Body('memory', type=int)
    #: VPC
    #: *Type:str*
    vpc = resource.Body('vpc')
    #: VPC_ID
    #: *Type:str*
    vpc_id = resource.Body('vpcid')
    #: Subnet name
    #: *Type:str*
    subnet = resource.Body('subnet')
    #: CIDR
    #: *Type:str*
    cidr = resource.Body('cidr')
    #: Cluster type
    #: possible values: (HA, Single)
    #: *Type:str*
    cluster_type = resource.Body('clustertype')
    #: Security group id
    #: when left empty on creation CCE will create one
    #: *Type:str*
    security_group_id = resource.Body('security_group_id')
    #: Endpoint
    #: *Type:str*
    endpoint = resource.Body('endpoint')
    #: External endpoint
    #: *Type:str*
    external_endpoint = resource.Body('external_endpoint')
    #: Cluster type
    #: *Type:str*
    type = resource.Body('clustertype')
    #: host list
    host_list = resource.Body('hostList', type=ClusterNodeList)
    #: Region (used for create cluster)
    region = resource.Body('region')
    #: Public IP ID or EIP ID (used for create cluster)
    publicip_id = resource.Body('publicip_id')


class Cluster(_base.Resource):
    base_path = '/clusters'

    resources_key = ''
    resource_key = ''

    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # Properties
    #: specification
    spec = resource.Body('spec', type=ClusterSpec)
    #: Cluster status
    status = resource.Body('clusterStatus', type=dict)

    @classmethod
    def new(cls, **kwargs):
        if 'kind' not in kwargs:
            kwargs['kind'] = 'cluster'
        if 'apiVersion' not in kwargs:
            kwargs['apiVersion'] = 'v1'
        metadata = kwargs.get('metadata', '')
        if 'name' in kwargs and not metadata:
            name = kwargs.pop('name', '')
            kwargs['metadata'] = {
                'name': name
            }
        return cls(_synchronized=False, **kwargs)

    @classmethod
    def _get_one_match(cls, name_or_id, results):
        """Given a list of results, return the match"""
        the_result = None
        for maybe_result in results:
            id_value = cls._get_id(maybe_result)
            name_value = maybe_result.metadata.name

            if (id_value == name_or_id) or (name_value == name_or_id):
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if the_result is None:
                    the_result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = (msg % (cls.__name__, name_or_id))
                    raise exceptions.DuplicateResource(msg)

        return the_result

    @staticmethod
    def _get_id(value):
        """If a value is a Resource, return the canonical ID

        This will return either the value specified by `id` or
        `alternate_id` in that order if `value` is a Resource.
        If `value` is anything other than a Resource, likely to
        be a string already representing an ID, it is returned.
        """
        if isinstance(value, resource.Resource):
            return value.metadata.id
        else:
            return value

    def __getattribute__(self, name):
        """Return an attribute on this instance

        This is mostly a pass-through except for a specialization on
        the 'id' name, as this can exist under a different name via the
        `alternate_id` argument to resource.Body.
        """
        if name == 'id' or name == 'name':
            if name in self._body:
                return self._body[name]
            else:
                try:
                    metadata = self._body['metadata']
                    if name == 'id':
                        if isinstance(metadata, dict):
                            return metadata['uuid']
                        elif isinstance(metadata, _base.Metadata):
                            return metadata._body[metadata._alternate_id()]
                    else:
                        if isinstance(metadata, dict):
                            return metadata['name']
                        elif isinstance(metadata, _base.Metadata):
                            return metadata.name
                except KeyError:
                    return None
        else:
            return object.__getattribute__(self, name)

    def delete_nodes(self, session, node_names, headers=None):
        """Delete nodes from the cluster by their name
        """
        nodes = []
        if isinstance(node_names, list):
            # Is given a list
            for node in node_names:
                nodes.append({'name': node})
        elif isinstance(node_names, str):
            # a single string, consider as a name of single host
            nodes.append({'name': node_names})
        message = {
            'hosts': nodes
        }

        # Build additional arguments to the DELETE call
        args = self._prepare_override_args(
            additional_headers=headers
        )

        url = utils.urljoin(self.base_path, self.id, 'hosts')
        session.delete(url, json=message, **args)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.swr.v2 import _proxy
from otcextensions.sdk.swr.v2 import organization
from otcextensions.sdk.swr.v2 import repository
from otcextensions.sdk.swr.v2 import domain


class TestSwrProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestSwrProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestSwrOrganization(TestSwrProxy):
    def test_organization_create(self):
        self.verify_create(self.proxy.create_organization,
                           organization.Organization,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_organization_delete(self):
        self.verify_delete(self.proxy.delete_organization,
                           organization.Organization, True)

    def test_organization_get(self):
        self.verify_get(self.proxy.get_organization,
                        organization.Organization)

    def test_organizations(self):
        self.verify_list(self.proxy.organizations,
                         organization.Organization)


class TestSwrOrganzationPermissions(TestSwrProxy):
    def test_organization_permission_create(self):
        self.verify_create(self.proxy.create_organization_permissions,
                           organization.Permission,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_organization_permission_delete(self):
        self._verify(
            mock_method='otcextensions.sdk.swr.v2._base.Resource.'
                        '_delete',
            test_method=self.proxy.delete_organization_permissions,
            method_kwargs={
                'namespace': 'space',
                'user_ids': ['resource_id'],
            },
            expected_args=[
                self.proxy,
                ['resource_id'],
                '/manage/namespaces/space/access'
            ]),

    def test_organization_permission_update(self):
        self.verify_update(self.proxy.update_organization_permissions,
                           organization.Permission,
                           method_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'user_auth': 1
                                   }
                               ],
                           },
                           method_args=[],
                           expected_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'auth': 1
                                   }
                               ],
                           })

    def test_organization_permissions(self):
        self.verify_list(self.proxy.organization_permissions,
                         organization.Permission,
                         method_kwargs={
                             'namespace': 'id',
                             'permissions': [
                                 {
                                     'user_id': '123',
                                     'user_name': 'test',
                                     'auth': 1
                                 }
                             ],
                         })


class TestSwrRepository(TestSwrProxy):
    def test_repository_create(self):
        self.verify_create(self.proxy.create_repository,
                           repository.Repository,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_repository_delete(self):
        self.verify_delete(self.proxy.delete_repository,
                           repository.Repository, True,
                           method_kwargs={
                               'repository': 'resource_id'
                           },
                           expected_kwargs={
                               'namespace': 'resource_id'
                           })

    def test_repository_get(self):
        self.verify_get(self.proxy.get_repository,
                        repository.Repository,
                        method_kwargs={
                            'repository': 'resource_id'
                        },
                        expected_kwargs={
                            'namespace': 'resource_id'
                        })

    def test_repositories(self):
        self.verify_list(self.proxy.repositories,
                         repository.Repository)


class TestSwrRepositoryPermissions(TestSwrProxy):
    def test_repository_permission_create(self):
        self.verify_create(self.proxy.create_repository_permissions,
                           repository.Permission,
                           method_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'user_auth': 1
                                   }
                               ],
                           },
                           method_args=[],
                           expected_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'auth': 1
                                   }
                               ],
                           })

    def test_repository_permission_delete(self):
        self._verify(
            mock_method='otcextensions.sdk.swr.v2._base.Resource.'
                        '_delete',
            test_method=self.proxy.delete_repository_permissions,
            method_kwargs={
                'namespace': 'space',
                'repository': 'repo',
                'user_ids': ['resource_id'],
            },
            expected_args=[
                self.proxy,
                ['resource_id'],
                '/manage/namespaces/space/repos/repo/access'
            ]),

    def test_repository_permission_update(self):
        self.verify_update(self.proxy.update_repository_permissions,
                           repository.Permission,
                           method_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'user_auth': 1
                                   }
                               ],
                           },
                           method_args=[],
                           expected_kwargs={
                               'namespace': 'id',
                               'permissions': [
                                   {
                                       'user_id': '123',
                                       'user_name': 'test',
                                       'auth': 1
                                   }
                               ],
                           })

    def test_repository_permissions(self):
        self.verify_list(self.proxy.repository_permissions,
                         repository.Permission,
                         method_kwargs={
                             'namespace': 'id',
                             'repository': 'repo',
                             'permissions': [
                                 {
                                     'user_id': '123',
                                     'user_name': 'test',
                                     'auth': 1
                                 }
                             ],
                         })


class TestSwrDomain(TestSwrProxy):
    def test_domain_create(self):
        self.verify_create(self.proxy.create_domain,
                           domain.Domain,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_domain_update(self):
        self.verify_update(self.proxy.update_domain,
                           domain.Domain,
                           method_args=[],
                           method_kwargs={
                               'namespace': 'space',
                               'repository': 'repo',
                               'access_domain': 'domain',
                               'description': 'updated',
                               'permit': 'read',
                               'deadline': 'forever'},
                           expected_kwargs={
                               'namespace': 'space',
                               'repository': 'repo',
                               'access_domain': 'domain',
                               'id': 'domain',
                               'description': 'updated',
                               'permit': 'read',
                               'deadline': 'forever'})

    def test_domain_delete(self):
        self.verify_delete(self.proxy.delete_domain,
                           domain.Domain,
                           ignore_missing=True,
                           method_kwargs={
                               'namespace': 'space',
                               'repository': 'resource_id',
                               'access_domain': 'domain',
                           },
                           expected_kwargs={
                               'repository': 'resource_id',
                               'id': 'domain',
                               'ignore_missing': True,
                               'namespace': 'space'
                           },
                           method_args=[])

    def test_domain_get(self):
        self.verify_get(self.proxy.get_domain,
                        domain.Domain,
                        method_kwargs={
                            'namespace': 'resource_id',
                            'repository': 'repo',
                            'access_domain': 'domain'
                        },
                        expected_kwargs={
                            'repository': 'repo',
                            'id': 'domain',
                            'namespace': 'resource_id'
                        },
                        method_args=[])

    def test_domains(self):
        self.verify_list(self.proxy.domains,
                         domain.Domain)


class TestExtractName(TestSwrProxy):

    def test_extract_name(self):
        self.assertEqual(
            [],
            self.proxy._extract_name('/v2')
        )

        self.assertEqual(
            ['manage', 'shared-repositories'],
            self.proxy._extract_name(
                '/v2/manage/shared-repositories')
        )
        self.assertEqual(
            ['manage', 'repos'],
            self.proxy._extract_name('/v2/manage/repos')
        )
        self.assertEqual(
            ['manage', 'namespaces'],
            self.proxy._extract_name('/v2/manage/namespaces')
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from otcextensions.osclient.vlb.v3 import load_balancer
from otcextensions.tests.unit.osclient.vlb.v3 import fakes


class TestLoadBalancer(fakes.TestVLB):

    def setUp(self):
        super(TestLoadBalancer, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeLoadBalancer.create_one()

        flat_data = load_balancer._flatten_loadbalancer(obj)

        data = (
            flat_data['availability_zone_list'],
            flat_data['created_at'],
            flat_data['description'],
            flat_data['deletion_protection_enable'],
            flat_data['is_guaranteed'],
            flat_data['is_admin_state_up'],
            flat_data['ip_target_enable'],
            flat_data['l4_flavor_id'],
            flat_data['l7_flavor_id'],
            flat_data['name'],
            flat_data['network_ids'],
            flat_data['subnet_type'],
            flat_data['operating_status'],
            flat_data['project_id'],
            flat_data['provider'],
            flat_data['provisioning_status'],
            flat_data['updated_at'],
            flat_data['ip_address'],
            flat_data['port_id'],
            flat_data['subnet_id'],
            flat_data['vpc_id']
        )

        cmp_data = (
            obj.availability_zones,
            obj.created_at,
            obj.description,
            obj.deletion_protection_enable,
            obj.is_guaranteed,
            obj.is_admin_state_up,
            obj.ip_target_enable,
            obj.l4_flavor_id,
            obj.l7_flavor_id,
            obj.name,
            obj.network_ids,
            obj.subnet_type,
            obj.operating_status,
            obj.project_id,
            obj.provider,
            obj.provisioning_status,
            obj.updated_at,
            obj.ip_address,
            obj.port_id,
            obj.subnet_id,
            obj.vpc_id
        )

        self.assertEqual(data, cmp_data)

    def test_add_tags_to_loadbalancer_output(self):
        obj = fakes.FakeLoadBalancer.create_one()

        column = ()
        data = ()
        verify_column = (
            'tags',
        )
        verify_data = (
            ('value=val-tags, key=key-tags',)
        )

        data, column = load_balancer._add_tags_to_load_balancer_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_normalize_tags(self):
        tags = [
            'key1=value',
            'key2=',
            'key3'
        ]

        verify_result = [
            {'key': 'key1', 'value': 'value'},
            {'key': 'key2', 'value': ''},
            {'key': 'key3', 'value': ''}
        ]

        result = load_balancer._normalize_tags(tags)

        self.assertEqual(result, verify_result)

    def test_add_eips_to_loadbalancer_output(self):
        obj = fakes.FakeLoadBalancer.create_one()

        column = ()
        data = ()
        verify_column = (
            'eip_id_1',
            'eip_address_1',
            'ip_version_1',
            'eip_id_2',
            'eip_address_2',
            'ip_version_2'
        )
        verify_data = (
            'eip-uuid-1',
            'eip-address-1',
            'ip-version-1',
            'eip-uuid-2',
            'eip-address-2',
            'ip-version-2'
        )

        data, column = load_balancer._add_eips_to_load_balancer_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_publicips_to_loadbalancer_output(self):
        obj = fakes.FakeLoadBalancer.create_one()

        column = ()
        data = ()
        verify_column = (
            'publicip_id_1',
            'publicip_address_1',
            'publicip_ip_version_1',
            'publicip_id_2',
            'publicip_address_2',
            'publicip_ip_version_2',
        )
        verify_data = (
            'publicip-id-1',
            'publicip-address-1',
            'ip-version-1',
            'publicip-id-2',
            'publicip-address-2',
            'ip-version-2',
        )

        data, column = load_balancer._add_publicips_to_load_balancer_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_pools_to_loadbalancer_output(self):
        obj = fakes.FakeLoadBalancer.create_one()

        column = ()
        data = ()
        verify_column = (
            'pool_id_1',
            'pool_id_2',
        )
        verify_data = (
            'pool-id-1',
            'pool-id-2',
        )

        data, column = load_balancer._add_pools_to_load_balancer_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_listeners_to_loadbalancer_output(self):
        obj = fakes.FakeLoadBalancer.create_one()

        column = ()
        data = ()
        verify_column = (
            'listener_id_1',
            'listener_id_2',
        )
        verify_data = (
            'listener-id-1',
            'listener-id-2',
        )

        data, column = load_balancer._add_listeners_to_load_balancer_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import _log

from otcextensions.sdk.dms.v1 import _base

_logger = _log.setup_logging('openstack')


class GroupMessage(_base.Resource):

    base_path = '/queues/%(queue_id)s/groups/%(consumer_group_id)s/messages'

    _query_mapping = resource.QueryParameters('max_msgs', 'time_wait')

    # Properties
    #: Queue id
    queue_id = resource.URI('queue_id')
    #: Consumer group id
    consumer_group_id = resource.URI('consumer_group_id')
    #: Message dict
    #: *Type: dict*
    message = resource.Body('message', type=dict)
    #: handler
    handler = resource.Body('handler')
    #: Status of the message
    status = resource.Body('status')
    #: Success number of the message
    #: *Type: int*
    success = resource.Body('success', type=int)
    #: Fail number of the message
    #: *Type: int*
    fail = resource.Body('fail', type=int)

    # use get method to consume message, return a list of self
    @classmethod
    def list(cls, session, paginated=False, **params):

        headers = {"Accept": "application/json",
                   "Content-type": "application/json"}
        uri = cls.base_path % params

        # NOTES: this API is so different from others, it's not a RESTFUL
        # style, allow user to pass mulitple tags as the query parameters
        # which can not leverage method of session directlly.

        query_params = cls._query_mapping._transpose(params, cls)
        resp = session.get(
            uri,
            headers=headers,
            params=query_params)

        if resp is not None:
            resp = resp.json()
            ret = []
            # resp is a list
            for r in resp:
                r['queue_id'] = params.get('queue_id')
                r['consumer_group_id'] = params.get('consumer_group_id')
                ret.append(cls.existing(**r))

            return ret

    def ack(self, session, status='success'):
        base_path = '/queues/%(queue_id)s/groups/%(consumer_group_id)s/ack'

        uri = base_path % self._uri.attributes

        body = {
            "message": [
                {
                    "handler": self.handler,
                    "status": self.status if self.status else status
                }
            ]
        }

        response = session.post(
            uri, json=body
        )

        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ExtraInfo(resource.Resource):
    #: Properties
    #: Backup name
    name = resource.Body('name')
    #: Backup description
    description = resource.Body('description')
    #: Number of days that backups can be retained
    retention_duration = resource.Body('retention_duration', type=int)


class CheckpointResource(resource.Resource):
    #: Properties
    #: Number of backups
    backup_count = resource.Body('backup_count')
    #: Backup size
    backup_size = resource.Body('backup_size')
    #: Extra information of the resource
    extra_info = resource.Body('extra_info')
    #: ID of the resource to be backed up
    id = resource.Body('id')
    #: Name of the resource to be backed up
    #: max 255 characters
    name = resource.Body('name')
    #: Protection status, choices:
    #: available, error, protecting, restoring, removing
    protect_status = resource.Body('protect_status')
    #: Allocated capacity for the associated resource in GB
    resource_size = resource.Body('resource_size')
    #: type of the resource to be backed up.
    type = resource.Body('type')


class SkippedResource(resource.Resource):
    #: Properties
    #: Error Codes
    code = resource.Body('code')
    #: Resource ID
    id = resource.Body('id')
    #: Resource name
    name = resource.Body('name')
    #: Reason for skipping
    reason = resource.Body('reason')
    #: type of the resource to be backed up.
    type = resource.Body('type')


class Vault(resource.Resource):
    #: Properties
    #: Vault ID
    id = resource.Body('id')
    #: Vault name
    name = resource.Body('name')
    #: Backup objects
    resources = resource.Body('resource', type=list,
                              list_type=CheckpointResource)
    #: Resources skipped during backup
    skipped_resources = resource.Body('skipped_resources', type=list,
                                      list_type=SkippedResource)


class IncludeVolumes(resource.Resource):
    #: Properties
    #: EVS disk UUID
    id = resource.Body('id')
    #: OS type
    os_version = resource.Body('os_version')


class ResourceExtraInfo(resource.Resource):
    #: Properties
    #: List of IDs that is excluded from backup.
    exclude_volumes = resource.Body('exclude_volumes', type=list)
    #: Disk to be backed up
    include_volumes = resource.Body('include_volumes', type=list,
                                    list_type=IncludeVolumes)


class ResourceDetails(resource.Resource):
    #: Checkpoint resource creation Properties
    #: Extra information of the resource
    extra_info = resource.Body('extra_info', type=ResourceExtraInfo)
    #: ID of the resource to be backed up
    id = resource.Body('id')
    #: Name of the resource to be backed up
    name = resource.Body('name')
    #: Type of the resource to be backed up
    resource_type = resource.Body('type')


class Parameters(resource.Resource):
    #: Properties
    #: Whether automatic triggering is enabled
    auto_trigger = resource.Body('auto_trigger', type=bool)
    #: Backup description
    description = resource.Body('description')
    #: Whether the backup is an incremental backup
    incremental = resource.Body('incremental')
    #: Backup name
    name = resource.Body('name')
    #: UUID list of resources to be backed up
    resources = resource.Body('resources', type=list)
    # Resource details
    resource_details = resource.Body('resource_details',
                                     type=ResourceDetails)


class Checkpoint(resource.Resource):
    """CBR Checkpoint Resource"""
    resource_key = 'checkpoint'
    resources_key = ''
    base_path = '/checkpoints'

    # capabilities
    allow_create = True
    allow_list = False
    allow_fetch = True
    allow_delete = False
    allow_commit = False

    _query_mapping = resource.QueryParameters(
        'checkpoint_id')

    #: Properties
    #: Creation time
    #: Example: 2020-02-05T10:38:34.209782
    created_at = resource.Body('created_at')
    #: Extra info
    extra_info = resource.Body('extra_info', type=ExtraInfo)
    #: Resoure point ID
    id = resource.Body('id')
    #: Checkpoint creation parameters
    parameters = resource.Body('parameters', type=Parameters)
    #: Project ID
    project_id = resource.Body('project_id')
    #: Status
    status = resource.Body('status')
    #: Vault information
    vault = resource.Body('vault', type=Vault)
    #: Vault ID for checkpoint creation
    vault_id = resource.Body('vault_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import service_cluster
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestListServiceClusters(fakes.TestModelartsv1):
    objects = fakes.FakeServiceCluster.create_multiple(3)

    column_list_headers = (
        "Cluster Id",
        "Cluster Name",
        "Created At",
        "Status",
        "Allocatable Resources",
        "Charging Mode",
        "Max Node Count",
        "Nodes",
        "Services Count",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.cluster_id,
                s.cluster_name,
                cli_utils.UnixTimestampFormatter(s.created_at),
                s.status,
                cli_utils.YamlFormat(s.allocatable_resources),
                s.charging_mode,
                s.max_node_count,
                cli_utils.YamlFormat(s.nodes),
                cli_utils.YamlFormat(s.services_count),
            )
        )

    def setUp(self):
        super(TestListServiceClusters, self).setUp()

        self.cmd = service_cluster.ListServiceClusters(self.app, None)

        self.client.service_clusters = mock.Mock()
        self.client.api_mock = self.client.service_clusters

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--cluster-name",
            "1",
            "--limit",
            "2",
            "--offset",
            "3",
            "--order",
            "asc",
            "--sort-by",
            "cluster_name",
            "--status",
            "6",
        ]

        verifylist = [
            ("cluster_name", "1"),
            ("limit", 2),
            ("offset", 3),
            ("order", "asc"),
            ("sort_by", "cluster_name"),
            ("status", "6"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            cluster_name="1",
            limit=2,
            offset=3,
            order="asc",
            sort_by="cluster_name",
            status="6",
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Formatters for cli outputs"""
import datetime

import yaml
from cliff import columns as cliff_columns


class literal(str):
    pass


def literal_presenter(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='>')


yaml.add_representer(literal, literal_presenter)


def scrub_dict(d):
    new_dict = {}
    for k, v in d.items():
        if isinstance(v, dict):
            v = scrub_dict(v)
        if v not in ('', None, {}):
            new_dict[k] = v
    return new_dict


def wrap_text(s):
    length = 100
    if isinstance(s, str) and len(s) >= length:
        return '\n'.join([s[i:i + length] for i in range(0, len(s), length)])
    return s


class WrapText(cliff_columns.FormattableColumn):
    def human_readable(self):
        return wrap_text(self._value)


class UnixTimestampFormatter(cliff_columns.FormattableColumn):
    """Generate a formatted string of a hostname."""

    def human_readable(self):
        if self._value is None:
            return ''

        # Convert Unix timestamp to GMT+1
        gmt_offset = 1
        if not isinstance(self._value, int):
            self._value = int(self._value) / 1000
        else:
            self._value = self._value / 1000
        timestamp = datetime.datetime.fromtimestamp(self._value)
        timezone_offset = datetime.timezone(
            datetime.timedelta(hours=gmt_offset)
        )
        timestamp_gmt = timestamp.astimezone(timezone_offset)

        # Format the timestamp as a string with the timezone offset
        timezone_offset_str = timestamp_gmt.strftime('%z')
        timezone_offset_formatted = (
            f'GMT{timezone_offset_str[:3]}:{timezone_offset_str[3:]}'
        )
        return (
            timestamp_gmt.strftime('%Y-%m-%d %H:%M:%S ')
            + timezone_offset_formatted
        )
        # Format GMT+1 timestamp as string


class YamlFormat(cliff_columns.FormattableColumn):
    def remove_null_values(self, data):
        if isinstance(data, dict):
            if (
                'record_delimiter' in data.keys()
                and '\n' in data['record_delimiter']
            ):
                data['record_delimiter'] = '\\n'
            return {
                k: self.remove_null_values(v)
                for k, v in data.items()
                if v is not None
            }
        elif isinstance(data, list):
            return [self.remove_null_values(v) for v in data if v is not None]
        else:
            return data

    def human_readable(self):
        data = self._value
        if type(data) not in (dict, list):
            data = self._value.to_dict(computed=False)
        data = self.remove_null_values(data)
        return yaml.dump(data, width=100)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestIpAddressGroup(TestVlb):

    def setUp(self):
        super(TestIpAddressGroup, self).setUp()
        self.create_ip_address_group()

    def test_01_list_ip_address_groups(self):
        ip_groups = list(self.client.ip_address_groups(
            name=TestIpAddressGroup.ip_address_group.name))
        self.assertGreaterEqual(len(ip_groups), 0)

    def test_02_get_ip_address_group(self):
        ip_address_group = self.client.get_ip_address_group(
            TestVlb.ip_address_group)
        self.assertIsNotNone(ip_address_group)

    def test_03_find_ip_address_group(self):
        ip_address_group = self.client.find_ip_address_group(
            TestVlb.ip_address_group.name)
        self.assertIsNotNone(ip_address_group)

    def test_04_update_ip_address_group(self):
        new_description = 'changed'
        ip_list = [{"ip": "192.168.1.125", "description": ""}]
        ip_address_group = self.client.update_ip_address_group(
            TestVlb.ip_address_group,
            description=new_description,
            ip_list=ip_list
        )
        self.assertEqual(ip_address_group['description'], new_description)
        self.assertEqual(ip_address_group['ip_list'], ip_list)

        # cleanup
        self.client.delete_ip_address_group(
            TestVlb.ip_address_group
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import exceptions


class ListDependenciesResult(resource.Resource):
    id = resource.Body('id', type=str)
    owner = resource.Body('owner', type=str)
    link = resource.Body('link', type=str)
    runtime = resource.Body('runtime', type=str)
    etag = resource.Body('etag', type=str)
    size = resource.Body('size', type=int)
    name = resource.Body('name', type=str)
    file_name = resource.Body('file_name', type=str)
    description = resource.Body('description', type=str)
    version = resource.Body('version', type=int)
    last_modified = resource.Body('last_modified', type=int)
    dep_id = resource.Body('dep_id', type=str)
    is_shared = resource.Body('is_shared', type=bool)


class Dependency(resource.Resource):
    base_path = '/fgs/dependencies'

    requires_id = False
    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'marker', 'maxitems', 'ispublic',
        'dependency_type', 'runtime', 'name', 'limit'
    )

    # Properties
    depend_file = resource.Body('depend_file', type=str)
    depend_link = resource.Body('depend_link', type=str)
    depend_type = resource.Body('depend_type', type=str)
    runtime = resource.Body('runtime', type=str)
    name = resource.Body('name', type=str)
    description = resource.Body('description', type=str)

    # Attributes
    #: Dependency list.
    dependencies = resource.Body(
        'dependencies', type=list, list_type=ListDependenciesResult
    )
    #: Next read location.
    next_marker = resource.Body('next_marker', type=int)
    #: Total number of dependencies.
    count = resource.Body('count', type=int)

    owner = resource.Body('owner', type=str)
    link = resource.Body('link', type=str)
    etag = resource.Body('etag', type=str)
    size = resource.Body('size', type=int)
    file_name = resource.Body('file_name', type=str)
    version = resource.Body('version', type=int)
    dep_id = resource.Body('dep_id', type=str)
    last_modified = resource.Body('last_modified', type=int)

    def _delete_version(self, session, dependency):
        """Delete Version
        """
        url = self.base_path + (f'/{dependency.dep_id}'
                                f'/version/{dependency.version}')
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
import typing as ty


class Resource(resource.Resource):

    def _prepare_request_body(
            self,
            patch,
            prepend_key,
            *,
            resource_request_key=None,
    ):
        body: ty.Union[ty.Dict[str, ty.Any], ty.List[ty.Any]]
        if not self._store_unknown_attrs_as_properties:
            # Default case
            body = self._body.dirty
        else:
            body = self._unpack_properties_to_resource_root(
                self._body.dirty
            )

        if prepend_key:
            if resource_request_key is not None:
                body = {resource_request_key: body}
            elif self.resource_key is not None:
                body = {self.resource_key: body}
        if 'permissions' in body:
            return body['permissions']
        return body

    def _delete(self, session, user_ids, base_path):
        """Delete permissions
        """
        response = session.delete(base_path, json=user_ids)
        exceptions.raise_from_response(response)
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.rds.v3 import _base


class Backup(_base.Resource):

    base_path = '/backups'
    resources_key = 'backups'
    resource_key = 'backup'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        'offset', 'begin_time', 'instance_id', 'id',
        'type', 'begin_time', 'end_time', 'offset', 'limit',
        id='backup_id', type='backup_type')

    #: Backup id
    #: Type: uuid*
    id = resource.Body('id')
    #: Begin time
    begin_time = resource.Body('begin_time')
    #: Create back of specific dbs
    #: *Type:list*
    databases = resource.Body('databases', type=list)
    #: Datastore
    #: *Type:dict*
    datastore = resource.Body('datastore', type=dict)
    #: Data backup description
    description = resource.Body('description')
    #: Instance id
    instance_id = resource.Body('instance_id')
    #: Back file size in GB
    #: *Type:int*
    size = resource.Body('size', type=int)
    #: Backup status
    status = resource.Body('status')
    #: Finished time
    end_time = resource.Body('end_time')
    #: Backup type
    #:  `auto`: automated full backup
    #:  `manual`: manual full backup
    #:  `fragment`: differential full backup
    #:  `incremental`: automated incremental backup
    type = resource.Body('type')

    def create(self, session, prepend_key=False, base_path=None):
        return super(Backup, self).create(session,
                                          prepend_key=prepend_key,
                                          base_path=base_path)

    def commit(self, session, prepend_key=False, **further_attrs):
        return super(Backup, self).commit(
            session,
            prepend_key=prepend_key,
            **further_attrs)

    def fetch(self, session, requires_id=True,
              base_path=None, error_message=None, **params):
        """Get a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param boolean requires_id: A boolean indicating whether resource ID
                                    should be part of the requested URI.
        :param str base_path: Base part of the URI for fetching resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param str error_message: An Error message to be returned if
                                  requested object does not exist.
        :param dict params: Additional parameters that can be consumed.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_fetch` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
                 the resource was not found.
        """
        if not self.allow_fetch:
            raise exceptions.MethodNotSupported(self, "fetch")

        # Create request parameters
        request_params = {
            'instance_id': self.instance_id,
            'backup_id': self.id
        }

        # Merge with additional params if provided
        request_params.update(params)

        query_params = self._query_mapping._transpose(request_params, self)
        url = utils.urljoin(self.base_path) % request_params

        session = self._get_session(session)
        microversion = self._get_microversion(session)
        response = session.get(
            url, microversion=microversion,
            headers={"Accept": "application/json"},
            params=query_params.copy())

        exceptions.raise_from_response(response)

        try:
            body = response.json()
            if self.resources_key in body:
                body = body[self.resources_key]

            if not len(body) == 1:
                raise exceptions.SDKException('Not a single result returned')

            body = body[0]
            body_attrs = self._consume_body_attrs(body)
            self._body.attributes.update(body_attrs)
            self._body.clean()

        except ValueError:
            # Server returned not parse-able response (202, 204, etc)
            # Do simply nothing
            pass

        dict.update(self, self.to_dict())

        return self


class BackupFile(resource.Resource):

    base_path = '/backup-files'
    resources_key = 'files'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters('backup_id')

    #:  Indicates the file name
    #:  *Type: string*
    name = resource.Body('name')
    #:  Indicates the file size in KB.
    #:  *Type: long*
    size = resource.Body('size', type=int)
    #:  Indicates the link for downloading the backup file.
    #:  *Type: string*
    download_link = resource.Body('download_link')
    #:  Indicates the link expiration time.
    #:  The format is "yyyy-mmddThh:mm:ssZ".
    #:  *Type: string*
    expires_at = resource.Body('link_expired_time')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from openstackclient.tests.unit import utils

from otcextensions.common import sdk_utils
from otcextensions.osclient.load_balancer.v1 import pool
from otcextensions.tests.unit.osclient.load_balancer.v1 import fakes


class TestListPool(fakes.TestLoadBalancer):

    _objects = fakes.FakePool.create_multiple(3)

    columns = (
        'id', 'name', 'project_id', 'provisioning_status',
        'protocol', 'lb_algorithm', 'admin_state_up')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.project_id,
            '',
            s.protocol,
            s.lb_algorithm,
            s.is_admin_state_up,
        ))

    def setUp(self):
        super(TestListPool, self).setUp()

        self.cmd = pool.ListPool(self.app, None)

        self.client.pools = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.pools.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.pools.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_list_filter_values(self):
        arglist = [
            '--name', 'name',
            '--protocol', 'TCP',
            '--lb_algorithm', 'ROUND_ROBIN',
            '--description', 'descr',
            '--load_balancer', 'lb',
        ]

        verifylist = [
            ('name', 'name'),
            ('protocol', 'TCP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('description', 'descr'),
            ('load_balancer', 'lb')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.pools.side_effect = [
            {}
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.pools.assert_called_once_with(
            name='name',
            protocol='TCP',
            lb_algorithm='ROUND_ROBIN',
            description='descr',
            load_balancer_id='lb'
        )

    def test_list_filter_exceptions_proto(self):
        arglist = [
            '--protocol', 'bad',
        ]

        verifylist = [
            ('protocol', 'bad'),
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_list_filter_exceptions_algo(self):
        arglist = [
            '--lb_algorithm', 'bad',
        ]

        verifylist = [
            ('lb_algorithm', 'bad'),
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestShowPool(fakes.TestLoadBalancer):

    _object = fakes.FakePool.create_one()

    columns = (
        'admin_state_up', 'description',
        'healthmonitor_id', 'id', 'lb_algorithm', 'listeners',
        'loadbalancers', 'member_ids', 'name', 'protocol',
        'provisioning_status', 'session_persistence')

    data = (
        _object.is_admin_state_up,
        _object.description,
        # sdk_utils.ListOfIdsColumnBR(_object.health_monitor_ids),
        _object.health_monitor_id,
        _object.id,
        _object.lb_algorithm,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        sdk_utils.ListOfIdsColumnBR(_object.load_balancer_ids),
        sdk_utils.ListOfIdsColumnBR(_object.member_ids),
        _object.name,
        _object.protocol,
        _object.status,
        _object.session_persistence,
    )

    def setUp(self):
        super(TestShowPool, self).setUp()

        self.cmd = pool.ShowPool(self.app, None)

        self.client.find_pool = mock.Mock()

    def test_show_default(self):
        arglist = [
            'pool_id'
        ]

        verifylist = [
            ('pool', 'pool_id')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_pool.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_pool.assert_called_once_with(
            name_or_id='pool_id',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreatePool(fakes.TestLoadBalancer):

    _object = fakes.FakePool.create_one()

    columns = (
        'admin_state_up', 'description',
        'healthmonitor_id', 'id', 'lb_algorithm', 'listeners',
        'loadbalancers', 'member_ids', 'name', 'protocol',
        'provisioning_status', 'session_persistence')

    data = (
        _object.is_admin_state_up,
        _object.description,
        _object.health_monitor_id,
        _object.id,
        _object.lb_algorithm,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        sdk_utils.ListOfIdsColumnBR(_object.load_balancer_ids),
        sdk_utils.ListOfIdsColumnBR(_object.member_ids),
        _object.name,
        _object.protocol,
        _object.status,
        _object.session_persistence,
    )

    def setUp(self):
        super(TestCreatePool, self).setUp()

        self.cmd = pool.CreatePool(self.app, None)

        self.client.create_pool = mock.Mock()

    def test_create_default(self):
        arglist = [
            '--protocol', 'HTTP',
            '--lb_algorithm', 'ROUND_ROBIN',
            '--listener_id', 'listener',
            '--name', 'name',
            '--description', 'descr',
            '--session_persistence', '{"a": "b"}',
            '--disable'
        ]

        verifylist = [
            ('protocol', 'HTTP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('listener_id', 'listener'),
            ('disable', True),
            ('name', 'name'),
            ('description', 'descr'),
            ('session_persistence', '{"a": "b"}'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_pool.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_pool.assert_called_once_with(
            lb_algorithm='ROUND_ROBIN',
            listener_id='listener',
            name='name',
            protocol='HTTP',
            session_persistence={'a': 'b'},
            is_admin_state_up=False,
            description='descr'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_with_lb(self):
        arglist = [
            '--protocol', 'HTTP',
            '--lb_algorithm', 'ROUND_ROBIN',
            '--loadbalancer_id', 'lb',
        ]

        verifylist = [
            ('protocol', 'HTTP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('loadbalancer_id', 'lb'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_pool.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_pool.assert_called_once_with(
            lb_algorithm='ROUND_ROBIN',
            loadbalancer_id='lb',
            protocol='HTTP',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_exclusive_group(self):
        arglist = [
            'HTTP',
            'ROUND_ROBIN',
            '--listener_id', 'listener',
            '--loadbalancer_id', 'loadbalancer_id',
            '--name', 'name',
            '--session_persistence', "{'a': 'b'}"
        ]

        verifylist = [
            ('protocol', 'HTTP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('listener_id', 'listener'),
            ('loadbalancer_id', 'loadbalancer_id'),
            ('name', 'name'),
            ('session_persistence', "{'a': 'b'}"),
        ]

        # Verify cm is raising exception due to the exclusive group
        self.assertRaises(
            utils.ParserException,
            self.check_parser,
            self.cmd, arglist, verifylist
        )


class TestUpdatePool(fakes.TestLoadBalancer):

    _object = fakes.FakePool.create_one()

    columns = (
        'admin_state_up', 'description',
        'healthmonitor_id', 'id', 'lb_algorithm', 'listeners',
        'loadbalancers', 'member_ids', 'name', 'protocol',
        'provisioning_status', 'session_persistence')

    data = (
        _object.is_admin_state_up,
        _object.description,
        _object.health_monitor_id,
        _object.id,
        _object.lb_algorithm,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        sdk_utils.ListOfIdsColumnBR(_object.load_balancer_ids),
        sdk_utils.ListOfIdsColumnBR(_object.member_ids),
        _object.name,
        _object.protocol,
        _object.status,
        _object.session_persistence,
    )

    def setUp(self):
        super(TestUpdatePool, self).setUp()

        self.cmd = pool.SetPool(self.app, None)

        self.client.update_pool = mock.Mock()

    def test_update_default(self):
        arglist = [
            'pool',
            '--lb_algorithm', 'ROUND_ROBIN',
            '--name', 'name',
            '--description', 'descr',
            '--session_persistence', '{"a": "b"}',
            '--disable'
        ]

        verifylist = [
            ('pool', 'pool'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('disable', True),
            ('name', 'name'),
            ('description', 'descr'),
            ('session_persistence', '{"a": "b"}'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_pool.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_pool.assert_called_once_with(
            lb_algorithm='ROUND_ROBIN',
            pool='pool',
            name='name',
            session_persistence={'a': 'b'},
            is_admin_state_up=False,
            description='descr'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestDeletePool(fakes.TestLoadBalancer):

    _object = fakes.FakePool.create_one()

    def setUp(self):
        super(TestDeletePool, self).setUp()

        self.cmd = pool.DeletePool(self.app, None)

        self.client.delete_pool = mock.Mock()
        self.client.find_pool = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'pool',
        ]

        verifylist = [
            ('pool', ['pool']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_pool.side_effect = [
            {}
        ]
        self.client.find_pool.side_effect = [
            self._object
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_pool.assert_called_once_with(
            pool=self._object.id,
            ignore_missing=False
        )
